@@ -1,24 +1,27 @@
 # frozen_string_literal: true
 
-require "bundler"
-require "bundler/vendored_thor"
+require_relative "vendored_thor"
 
 module Bundler
   class CLI < Thor
-    require "bundler/cli/common"
+    require_relative "cli/common"
 
     package_name "Bundler"
 
     AUTO_INSTALL_CMDS = %w[show binstubs outdated exec open console licenses clean].freeze
-    PARSEABLE_COMMANDS = %w[
-      check config help exec platform show version
-    ].freeze
+    PARSEABLE_COMMANDS = %w[check config help exec platform show version].freeze
+
+    COMMAND_ALIASES = {
+      "check" => "c",
+      "install" => "i",
+      "list" => "ls",
+      "exec" => ["e", "ex", "exe"],
+      "cache" => ["package", "pack"],
+      "version" => ["-v", "--version"],
+    }.freeze
 
     def self.start(*)
       super
-    rescue Exception => e
-      Bundler.ui = UI::Shell.new
-      raise e
     ensure
       Bundler::SharedHelpers.print_major_deprecations!
     end
@@ -30,13 +33,31 @@ module Bundler
       end
     end
 
+    def self.all_aliases
+      @all_aliases ||= begin
+                         command_aliases = {}
+
+                         COMMAND_ALIASES.each do |name, aliases|
+                           Array(aliases).each do |one_alias|
+                             command_aliases[one_alias] = name
+                           end
+                         end
+
+                         command_aliases
+                       end
+    end
+
+    def self.aliases_for(command_name)
+      COMMAND_ALIASES.select {|k, _| k == command_name }.invert
+    end
+
     def initialize(*args)
       super
 
       custom_gemfile = options[:gemfile] || Bundler.settings[:gemfile]
       if custom_gemfile && !custom_gemfile.empty?
         Bundler::SharedHelpers.set_env "BUNDLE_GEMFILE", File.expand_path(custom_gemfile)
-        Bundler.reset_paths!
+        Bundler.reset_settings_and_root!
       end
 
       Bundler.settings.set_command_option_if_given :retry, options[:retry]
@@ -61,11 +82,6 @@ module Bundler
       end
     end
 
-    def self.deprecated_option(*args, &blk)
-      return if Bundler.feature_flag.forget_cli_options?
-      method_option(*args, &blk)
-    end
-
     check_unknown_options!(:except => [:config, :exec])
     stop_on_unknown_option! :exec
 
@@ -74,9 +90,7 @@ module Bundler
       version
       Bundler.ui.info "\n"
 
-      primary_commands = ["install", "update",
-                          Bundler.feature_flag.cache_command_is_package? ? "cache" : "package",
-                          "exec", "config", "help"]
+      primary_commands = ["install", "update", "cache", "exec", "config", "help"]
 
       list = self.class.printable_commands(true)
       by_name = list.group_by {|name, _message| name.match(/^bundle (\w+)/)[1] }
@@ -108,16 +122,20 @@ module Bundler
       else command = "bundle-#{cli}"
       end
 
-      man_path  = File.expand_path("../../../man", __FILE__)
-      man_pages = Hash[Dir.glob(File.join(man_path, "*")).grep(/.*\.\d*\Z/).collect do |f|
+      man_path = File.expand_path("../../../man", __FILE__)
+      # man files are located under ruby's mandir with the default gems of bundler
+      man_path = RbConfig::CONFIG["mandir"] unless File.directory?(man_path)
+      man_pages = Hash[Dir.glob(File.join(man_path, "**", "*")).grep(/.*\.\d*\Z/).collect do |f|
         [File.basename(f, ".*"), f]
       end]
 
       if man_pages.include?(command)
+        man_page = man_pages[command]
         if Bundler.which("man") && man_path !~ %r{^file:/.+!/META-INF/jruby.home/.+}
-          Kernel.exec "man #{man_pages[command]}"
+          Kernel.exec "man #{man_page}"
         else
-          puts File.read("#{man_path}/#{File.basename(man_pages[command])}.txt")
+          fallback_man_path = File.expand_path("../man", __FILE__)
+          puts File.read("#{fallback_man_path}/#{File.basename(man_page)}.ronn")
         end
       elsif command_path = Bundler.which("bundler-#{cli}")
         Kernel.exec(command_path, "--help")
@@ -142,9 +160,9 @@ module Bundler
       Gemfile to a gem with a gemspec, the --gemspec option will automatically add each
       dependency listed in the gemspec file to the newly created Gemfile.
     D
-    deprecated_option "gemspec", :type => :string, :banner => "Use the specified .gemspec to create the Gemfile"
+    method_option "gemspec", :type => :string, :banner => "Use the specified .gemspec to create the Gemfile"
     def init
-      require "bundler/cli/init"
+      require_relative "cli/init"
       Init.new(options.dup).run
     end
 
@@ -160,12 +178,15 @@ module Bundler
       "Use the specified gemfile instead of Gemfile"
     method_option "path", :type => :string, :banner =>
       "Specify a different path than the system default ($BUNDLE_PATH or $GEM_HOME).#{" Bundler will remember this value for future installs on this machine" unless Bundler.feature_flag.forget_cli_options?}"
-    map "c" => "check"
     def check
-      require "bundler/cli/check"
+      remembered_flag_deprecation("path")
+
+      require_relative "cli/check"
       Check.new(options).run
     end
 
+    map aliases_for("check")
+
     desc "remove [GEM [GEM ...]]", "Removes gems from the Gemfile"
     long_desc <<-D
       Removes the given gems from the Gemfile while ensuring that the resulting Gemfile is still valid. If the gem is not found, Bundler prints a error message and if gem could not be removed due to any reason Bundler will display a warning.
@@ -173,7 +194,7 @@ module Bundler
     method_option "install", :type => :boolean, :banner =>
       "Runs 'bundle install' after removing the gems from the Gemfile"
     def remove(*gems)
-      require "bundler/cli/remove"
+      require_relative "cli/remove"
       Remove.new(gems, options).run
     end
 
@@ -188,13 +209,13 @@ module Bundler
 
       If the bundle has already been installed, bundler will tell you so and then exit.
     D
-    deprecated_option "binstubs", :type => :string, :lazy_default => "bin", :banner =>
+    method_option "binstubs", :type => :string, :lazy_default => "bin", :banner =>
       "Generate bin stubs for bundled gems to ./bin"
-    deprecated_option "clean", :type => :boolean, :banner =>
+    method_option "clean", :type => :boolean, :banner =>
       "Run bundle clean automatically after install"
-    deprecated_option "deployment", :type => :boolean, :banner =>
+    method_option "deployment", :type => :boolean, :banner =>
       "Install using defaults tuned for deployment environments"
-    deprecated_option "frozen", :type => :boolean, :banner =>
+    method_option "frozen", :type => :boolean, :banner =>
       "Do not allow the Gemfile.lock to be updated after this install"
     method_option "full-index", :type => :boolean, :banner =>
       "Fall back to using the single-file index of all gems"
@@ -204,38 +225,46 @@ module Bundler
       "Specify the number of jobs to run in parallel"
     method_option "local", :type => :boolean, :banner =>
       "Do not attempt to fetch gems remotely and use the gem cache instead"
-    deprecated_option "no-cache", :type => :boolean, :banner =>
+    method_option "no-cache", :type => :boolean, :banner =>
       "Don't update the existing gem cache."
     method_option "redownload", :type => :boolean, :aliases => "--force", :banner =>
       "Force downloading every gem."
-    deprecated_option "no-prune", :type => :boolean, :banner =>
+    method_option "no-prune", :type => :boolean, :banner =>
       "Don't remove stale gems from the cache."
-    deprecated_option "path", :type => :string, :banner =>
-      "Specify a different path than the system default ($BUNDLE_PATH or $GEM_HOME). Bundler will remember this value for future installs on this machine"
+    method_option "path", :type => :string, :banner =>
+      "Specify a different path than the system default ($BUNDLE_PATH or $GEM_HOME).#{" Bundler will remember this value for future installs on this machine" unless Bundler.feature_flag.forget_cli_options?}"
     method_option "quiet", :type => :boolean, :banner =>
       "Only output warnings and errors."
-    deprecated_option "shebang", :type => :string, :banner =>
+    method_option "shebang", :type => :string, :banner =>
       "Specify a different shebang executable name than the default (usually 'ruby')"
     method_option "standalone", :type => :array, :lazy_default => [], :banner =>
       "Make a bundle that can work without the Bundler runtime"
-    deprecated_option "system", :type => :boolean, :banner =>
+    method_option "system", :type => :boolean, :banner =>
       "Install to the system location ($BUNDLE_PATH or $GEM_HOME) even if the bundle was previously installed somewhere else for this application"
     method_option "trust-policy", :alias => "P", :type => :string, :banner =>
       "Gem trust policy (like gem install -P). Must be one of " +
         Bundler.rubygems.security_policy_keys.join("|")
-    deprecated_option "without", :type => :array, :banner =>
+    method_option "without", :type => :array, :banner =>
       "Exclude gems that are part of the specified named group."
-    deprecated_option "with", :type => :array, :banner =>
+    method_option "with", :type => :array, :banner =>
       "Include gems that are part of the specified named group."
-    map "i" => "install"
     def install
-      SharedHelpers.major_deprecation(3, "The `--force` option has been renamed to `--redownload`") if ARGV.include?("--force")
-      require "bundler/cli/install"
+      SharedHelpers.major_deprecation(2, "The `--force` option has been renamed to `--redownload`") if ARGV.include?("--force")
+
+      %w[clean deployment frozen no-prune path shebang system without with].each do |option|
+        remembered_flag_deprecation(option)
+      end
+
+      remembered_negative_flag_deprecation("no-deployment")
+
+      require_relative "cli/install"
       Bundler.settings.temporary(:no_install => false) do
         Install.new(options.dup).run
       end
     end
 
+    map aliases_for("install")
+
     desc "update [OPTIONS]", "Update the current environment"
     long_desc <<-D
       Update will install the newest versions of the gems listed in the Gemfile. Use
@@ -275,63 +304,64 @@ module Bundler
     method_option "all", :type => :boolean, :banner =>
       "Update everything."
     def update(*gems)
-      SharedHelpers.major_deprecation(3, "The `--force` option has been renamed to `--redownload`") if ARGV.include?("--force")
-      require "bundler/cli/update"
+      SharedHelpers.major_deprecation(2, "The `--force` option has been renamed to `--redownload`") if ARGV.include?("--force")
+      require_relative "cli/update"
       Bundler.settings.temporary(:no_install => false) do
         Update.new(options, gems).run
       end
     end
 
-    desc "show GEM [OPTIONS]", "Shows all gems that are part of the bundle, or the path to a given gem"
-    long_desc <<-D
-      Show lists the names and versions of all gems that are required by your Gemfile.
-      Calling show with [GEM] will list the exact location of that gem on your machine.
-    D
-    method_option "paths", :type => :boolean,
-                           :banner => "List the paths of all gems that are required by your Gemfile."
-    method_option "outdated", :type => :boolean,
-                              :banner => "Show verbose output including whether gems are outdated."
-    def show(gem_name = nil)
-      if ARGV[0] == "show"
-        rest = ARGV[1..-1]
+    unless Bundler.feature_flag.bundler_3_mode?
+      desc "show GEM [OPTIONS]", "Shows all gems that are part of the bundle, or the path to a given gem"
+      long_desc <<-D
+        Show lists the names and versions of all gems that are required by your Gemfile.
+        Calling show with [GEM] will list the exact location of that gem on your machine.
+      D
+      method_option "paths", :type => :boolean,
+                             :banner => "List the paths of all gems that are required by your Gemfile."
+      method_option "outdated", :type => :boolean,
+                                :banner => "Show verbose output including whether gems are outdated."
+      def show(gem_name = nil)
+        if ARGV[0] == "show"
+          rest = ARGV[1..-1]
 
-        new_command = rest.find {|arg| !arg.start_with?("--") } ? "info" : "list"
+          if flag = rest.find{|arg| ["--verbose", "--outdated"].include?(arg) }
+            Bundler::SharedHelpers.major_deprecation(2, "the `#{flag}` flag to `bundle show` was undocumented and will be removed without replacement")
+          else
+            new_command = rest.find {|arg| !arg.start_with?("--") } ? "info" : "list"
 
-        new_arguments = rest.map do |arg|
-          next arg if arg != "--paths"
-          next "--path" if new_command == "info"
+            new_arguments = rest.map do |arg|
+              next arg if arg != "--paths"
+              next "--path" if new_command == "info"
+            end
+
+            old_argv = ARGV.join(" ")
+            new_argv = [new_command, *new_arguments.compact].join(" ")
+
+            Bundler::SharedHelpers.major_deprecation(2, "use `bundle #{new_argv}` instead of `bundle #{old_argv}`")
+          end
         end
-
-        old_argv = ARGV.join(" ")
-        new_argv = [new_command, *new_arguments.compact].join(" ")
-
-        Bundler::SharedHelpers.major_deprecation(3, "use `bundle #{new_argv}` instead of `bundle #{old_argv}`")
+        require_relative "cli/show"
+        Show.new(options, gem_name).run
       end
-      require "bundler/cli/show"
-      Show.new(options, gem_name).run
     end
-    # TODO: 2.0 remove `bundle show`
 
-    if Bundler.feature_flag.list_command?
-      desc "list", "List all gems in the bundle"
-      method_option "name-only", :type => :boolean, :banner => "print only the gem names"
-      method_option "only-group", :type => :string, :banner => "print gems from a particular group"
-      method_option "without-group", :type => :string, :banner => "print all gems expect from a group"
-      method_option "paths", :type => :boolean, :banner => "print the path to each gem in the bundle"
-      def list
-        require "bundler/cli/list"
-        List.new(options).run
-      end
-
-      map %w[ls] => "list"
-    else
-      map %w[list] => "show"
+    desc "list", "List all gems in the bundle"
+    method_option "name-only", :type => :boolean, :banner => "print only the gem names"
+    method_option "only-group", :type => :array, :default => [], :banner => "print gems from a given set of groups"
+    method_option "without-group", :type => :array, :default => [], :banner => "print all gems except from a given set of groups"
+    method_option "paths", :type => :boolean, :banner => "print the path to each gem in the bundle"
+    def list
+      require_relative "cli/list"
+      List.new(options).run
     end
 
+    map aliases_for("list")
+
     desc "info GEM [OPTIONS]", "Show information for the given gem"
     method_option "path", :type => :boolean, :banner => "Print full path to gem"
     def info(gem_name)
-      require "bundler/cli/info"
+      require_relative "cli/info"
       Info.new(options, gem_name).run
     end
 
@@ -351,8 +381,10 @@ module Bundler
       "Make binstubs that can work without the Bundler runtime"
     method_option "all", :type => :boolean, :banner =>
       "Install binstubs for all gems"
+    method_option "all-platforms", :type => :boolean, :default => false, :banner =>
+      "Install binstubs for all platforms"
     def binstubs(*gems)
-      require "bundler/cli/binstubs"
+      require_relative "cli/binstubs"
       Binstubs.new(options, gems).run
     end
 
@@ -363,12 +395,14 @@ module Bundler
     method_option "version", :aliases => "-v", :type => :string
     method_option "group", :aliases => "-g", :type => :string
     method_option "source", :aliases => "-s", :type => :string
+    method_option "git", :type => :string
+    method_option "branch", :type => :string
     method_option "skip-install", :type => :boolean, :banner =>
       "Adds gem to the Gemfile but does not install it"
     method_option "optimistic", :type => :boolean, :banner => "Adds optimistic declaration of version to gem"
     method_option "strict", :type => :boolean, :banner => "Adds strict declaration of version to gem"
     def add(*gems)
-      require "bundler/cli/add"
+      require_relative "cli/add"
       Add.new(options.dup, gems).run
     end
 
@@ -388,9 +422,10 @@ module Bundler
       "Do not attempt to fetch gems remotely and use the gem cache instead"
     method_option "pre", :type => :boolean, :banner => "Check for newer pre-release gems"
     method_option "source", :type => :array, :banner => "Check against a specific source"
-    method_option "strict", :type => :boolean, :banner =>
+    strict_is_update = Bundler.feature_flag.forget_cli_options?
+    method_option "filter-strict", :type => :boolean, :aliases => strict_is_update ? [] : %w[--strict], :banner =>
       "Only list newer versions allowed by your Gemfile requirements"
-    method_option "update-strict", :type => :boolean, :banner =>
+    method_option "update-strict", :type => :boolean, :aliases => strict_is_update ? %w[--strict] : [], :banner =>
       "Strict conservative resolution, do not allow any gem to be updated past latest --patch | --minor | --major"
     method_option "minor", :type => :boolean, :banner => "Prefer updating only to next minor version"
     method_option "major", :type => :boolean, :banner => "Prefer updating to next major version (default)"
@@ -403,53 +438,51 @@ module Bundler
     method_option "only-explicit", :type => :boolean, :banner =>
       "Only list gems specified in your Gemfile, not their dependencies"
     def outdated(*gems)
-      require "bundler/cli/outdated"
+      require_relative "cli/outdated"
       Outdated.new(options, gems).run
     end
 
-    if Bundler.feature_flag.cache_command_is_package?
-      map %w[cache] => :package
-    else
-      desc "cache [OPTIONS]", "Cache all the gems to vendor/cache", :hide => true
-      unless Bundler.feature_flag.cache_command_is_package?
-        method_option "all", :type => :boolean,
-                             :banner => "Include all sources (including path and git)."
-      end
-      method_option "all-platforms", :type => :boolean, :banner => "Include gems for all platforms present in the lockfile, not only the current one"
-      method_option "no-prune", :type => :boolean, :banner => "Don't remove stale gems from the cache."
-      def cache
-        require "bundler/cli/cache"
-        Cache.new(options).run
-      end
+    desc "fund [OPTIONS]", "Lists information about gems seeking funding assistance"
+    method_option "group", :aliases => "-g", :type => :array, :banner =>
+      "Fetch funding information for a specific group"
+    def fund
+      require_relative "cli/fund"
+      Fund.new(options).run
     end
 
-    desc "#{Bundler.feature_flag.cache_command_is_package? ? :cache : :package} [OPTIONS]", "Locks and then caches all of the gems into vendor/cache"
-    unless Bundler.feature_flag.cache_command_is_package?
-      method_option "all",  :type => :boolean,
-                            :banner => "Include all sources (including path and git)."
-    end
+    desc "cache [OPTIONS]", "Locks and then caches all of the gems into vendor/cache"
+    method_option "all",  :type => :boolean,
+                          :default => Bundler.feature_flag.cache_all?,
+                          :banner => "Include all sources (including path and git)."
     method_option "all-platforms", :type => :boolean, :banner => "Include gems for all platforms present in the lockfile, not only the current one"
     method_option "cache-path", :type => :string, :banner =>
       "Specify a different cache path than the default (vendor/cache)."
     method_option "gemfile", :type => :string, :banner => "Use the specified gemfile instead of Gemfile"
-    method_option "no-install", :type => :boolean, :banner => "Don't install the gems, only the package."
+    method_option "no-install", :type => :boolean, :banner => "Don't install the gems, only update the cache."
     method_option "no-prune", :type => :boolean, :banner => "Don't remove stale gems from the cache."
     method_option "path", :type => :string, :banner =>
-      "Specify a different path than the system default ($BUNDLE_PATH or $GEM_HOME). Bundler will remember this value for future installs on this machine"
+      "Specify a different path than the system default ($BUNDLE_PATH or $GEM_HOME).#{" Bundler will remember this value for future installs on this machine" unless Bundler.feature_flag.forget_cli_options?}"
     method_option "quiet", :type => :boolean, :banner => "Only output warnings and errors."
     method_option "frozen", :type => :boolean, :banner =>
-      "Do not allow the Gemfile.lock to be updated after this package operation's install"
+      "Do not allow the Gemfile.lock to be updated after this bundle cache operation's install"
     long_desc <<-D
-      The package command will copy the .gem files for every gem in the bundle into the
+      The cache command will copy the .gem files for every gem in the bundle into the
       directory ./vendor/cache. If you then check that directory into your source
       control repository, others who check out your source will be able to install the
       bundle without having to download any additional gems.
     D
-    def package
-      require "bundler/cli/package"
-      Package.new(options).run
+    def cache
+      SharedHelpers.major_deprecation 2,
+        "The `--all` flag is deprecated because it relies on being " \
+        "remembered across bundler invocations, which bundler will no longer " \
+        "do in future versions. Instead please use `bundle config set cache_all true`, " \
+        "and stop using this flag" if ARGV.include?("--all")
+
+      require_relative "cli/cache"
+      Cache.new(options).run
     end
-    map %w[pack] => :package
+
+    map aliases_for("cache")
 
     desc "exec [OPTIONS]", "Run the command in context of the bundle"
     method_option :keep_file_descriptors, :type => :boolean, :default => false
@@ -459,12 +492,13 @@ module Bundler
       bundle exec you can require and call the bundled gems as if they were installed
       into the system wide RubyGems repository.
     D
-    map "e" => "exec"
     def exec(*args)
-      require "bundler/cli/exec"
+      require_relative "cli/exec"
       Exec.new(options, args).run
     end
 
+    map aliases_for("exec")
+
     desc "config NAME [VALUE]", "Retrieve or set a configuration value"
     long_desc <<-D
       Retrieves or sets a configuration value. If only one parameter is provided, retrieve the value. If two parameters are provided, replace the
@@ -477,22 +511,19 @@ module Bundler
       will show the current value, as well as any superceded values and
       where they were specified.
     D
-    method_option "parseable", :type => :boolean, :banner => "Use minimal formatting for more parseable output"
-    def config(*args)
-      require "bundler/cli/config"
-      Config.new(options, args, self).run
-    end
+    require_relative "cli/config"
+    subcommand "config", Config
 
     desc "open GEM", "Opens the source directory of the given bundled gem"
     def open(name)
-      require "bundler/cli/open"
+      require_relative "cli/open"
       Open.new(options, name).run
     end
 
-    if Bundler.feature_flag.console_command?
+    unless Bundler.feature_flag.bundler_3_mode?
       desc "console [GROUP]", "Opens an IRB session with the bundle pre-loaded"
       def console(group = nil)
-        require "bundler/cli/console"
+        require_relative "cli/console"
         Console.new(options, group).run
       end
     end
@@ -510,7 +541,8 @@ module Bundler
         Bundler.ui.info "Bundler version #{Bundler::VERSION}#{build_info}"
       end
     end
-    map %w[-v --version] => :version
+
+    map aliases_for("version")
 
     desc "licenses", "Prints the license of all gems in the bundle"
     def licenses
@@ -526,7 +558,7 @@ module Bundler
       end
     end
 
-    if Bundler.feature_flag.viz_command?
+    unless Bundler.feature_flag.bundler_3_mode?
       desc "viz [OPTIONS]", "Generates a visual dependency graph", :hide => true
       long_desc <<-D
         Viz generates a PNG file of the current Gemfile as a dependency graph.
@@ -539,8 +571,8 @@ module Bundler
       method_option :version, :type => :boolean, :default => false, :aliases => "-v", :desc => "Set to show each gem version."
       method_option :without, :type => :array, :default => [], :aliases => "-W", :banner => "GROUP[ GROUP...]", :desc => "Exclude gems that are part of the specified named group."
       def viz
-        SharedHelpers.major_deprecation 3, "The `viz` command has been moved to the `bundle-viz` gem, see https://github.com/bundler/bundler-viz"
-        require "bundler/cli/viz"
+        SharedHelpers.major_deprecation 2, "The `viz` command has been moved to the `bundle-viz` gem, see https://github.com/bundler/bundler-viz"
+        require_relative "cli/viz"
         Viz.new(options.dup).run
       end
     end
@@ -549,14 +581,19 @@ module Bundler
 
     desc "gem NAME [OPTIONS]", "Creates a skeleton for creating a rubygem"
     method_option :exe, :type => :boolean, :default => false, :aliases => ["--bin", "-b"], :desc => "Generate a binary executable for your library."
-    method_option :coc, :type => :boolean, :desc => "Generate a code of conduct file. Set a default with `bundle config gem.coc true`."
+    method_option :coc, :type => :boolean, :desc => "Generate a code of conduct file. Set a default with `bundle config set --global gem.coc true`."
     method_option :edit, :type => :string, :aliases => "-e", :required => false, :banner => "EDITOR",
                          :lazy_default => [ENV["BUNDLER_EDITOR"], ENV["VISUAL"], ENV["EDITOR"]].find {|e| !e.nil? && !e.empty? },
                          :desc => "Open generated gemspec in the specified editor (defaults to $EDITOR or $BUNDLER_EDITOR)"
     method_option :ext, :type => :boolean, :default => false, :desc => "Generate the boilerplate for C extension code"
-    method_option :mit, :type => :boolean, :desc => "Generate an MIT license file. Set a default with `bundle config gem.mit true`."
-    method_option :test, :type => :string, :lazy_default => "rspec", :aliases => "-t", :banner => "rspec",
-                         :desc => "Generate a test directory for your library, either rspec or minitest. Set a default with `bundle config gem.test rspec`."
+    method_option :git, :type => :boolean, :default => true, :desc => "Initialize a git repo inside your library."
+    method_option :mit, :type => :boolean, :desc => "Generate an MIT license file. Set a default with `bundle config set --global gem.mit true`."
+    method_option :rubocop, :type => :boolean, :desc => "Add rubocop to the generated Rakefile and gemspec. Set a default with `bundle config set --global gem.rubocop true`."
+    method_option :test, :type => :string, :lazy_default => Bundler.settings["gem.test"] || "", :aliases => "-t", :banner => "Use the specified test framework for your library",
+                         :desc => "Generate a test directory for your library, either rspec, minitest or test-unit. Set a default with `bundle config set --global gem.test (rspec|minitest|test-unit)`."
+    method_option :ci, :type => :string, :lazy_default => Bundler.settings["gem.ci"] || "",
+                       :desc => "Generate CI configuration, either GitHub Actions, Travis CI, GitLab CI or CircleCI. Set a default with `bundle config set --global gem.ci (github|travis|gitlab|circle)`"
+
     def gem(name)
     end
 
@@ -564,7 +601,7 @@ module Bundler
       def gem_command.run(instance, args = [])
         arity = 1 # name
 
-        require "bundler/cli/gem"
+        require_relative "cli/gem"
         cmd_args = args + [instance]
         cmd_args.unshift(instance.options)
 
@@ -592,7 +629,7 @@ module Bundler
     method_option "force", :type => :boolean, :default => false, :banner =>
       "Forces clean even if --path is not set"
     def clean
-      require "bundler/cli/clean"
+      require_relative "cli/clean"
       Clean.new(options.dup).run
     end
 
@@ -600,7 +637,7 @@ module Bundler
     method_option "ruby", :type => :boolean, :default => false, :banner =>
       "only display ruby related platform information"
     def platform
-      require "bundler/cli/platform"
+      require_relative "cli/platform"
       Platform.new(options).run
     end
 
@@ -610,8 +647,8 @@ module Bundler
     method_option "group", :type => :string, :banner =>
      "Install gem into a bundler group"
     def inject(name, version)
-      SharedHelpers.major_deprecation 3, "The `inject` command has been replaced by the `add` command"
-      require "bundler/cli/inject"
+      SharedHelpers.major_deprecation 2, "The `inject` command has been replaced by the `add` command"
+      require_relative "cli/inject"
       Inject.new(options.dup, name, version).run
     end
 
@@ -622,6 +659,8 @@ module Bundler
       "do not attempt to fetch remote gemspecs and use the local gem cache only"
     method_option "print", :type => :boolean, :default => false, :banner =>
       "print the lockfile to STDOUT instead of writing to the file system"
+    method_option "gemfile", :type => :string, :banner =>
+      "Use the specified gemfile instead of Gemfile"
     method_option "lockfile", :type => :string, :default => nil, :banner =>
       "the path the lockfile should be written to"
     method_option "full-index", :type => :boolean, :default => false, :banner =>
@@ -641,7 +680,7 @@ module Bundler
     method_option "conservative", :type => :boolean, :banner =>
       "If updating, use bundle install conservative update behavior and do not allow shared dependencies to be updated"
     def lock
-      require "bundler/cli/lock"
+      require_relative "cli/lock"
       Lock.new(options).run
     end
 
@@ -661,13 +700,13 @@ module Bundler
     method_option "quiet", :type => :boolean, :banner =>
         "Only output warnings and errors."
     def doctor
-      require "bundler/cli/doctor"
+      require_relative "cli/doctor"
       Doctor.new(options).run
     end
 
     desc "issue", "Learn how to report an issue in Bundler"
     def issue
-      require "bundler/cli/issue"
+      require_relative "cli/issue"
       Issue.new.run
     end
 
@@ -678,12 +717,12 @@ module Bundler
       checkout --force`.
     D
     def pristine(*gems)
-      require "bundler/cli/pristine"
+      require_relative "cli/pristine"
       Pristine.new(gems).run
     end
 
     if Bundler.feature_flag.plugins?
-      require "bundler/cli/plugin"
+      require_relative "cli/plugin"
       desc "plugin", "Manage the bundler plugins"
       subcommand "plugin", Plugin
     end
@@ -691,12 +730,17 @@ module Bundler
     # Reformat the arguments passed to bundle that include a --help flag
     # into the corresponding `bundle help #{command}` call
     def self.reformatted_help_args(args)
-      bundler_commands = all_commands.keys
+      bundler_commands = (COMMAND_ALIASES.keys + COMMAND_ALIASES.values).flatten
+
       help_flags = %w[--help -h]
-      exec_commands = %w[e ex exe exec]
+      exec_commands = ["exec"] + COMMAND_ALIASES["exec"]
+
       help_used = args.index {|a| help_flags.include? a }
       exec_used = args.index {|a| exec_commands.include? a }
+
       command = args.find {|a| bundler_commands.include? a }
+      command = all_aliases[command] if all_aliases[command]
+
       if exec_used && help_used
         if exec_used + help_used == 1
           %w[help exec]
@@ -712,11 +756,11 @@ module Bundler
       end
     end
 
-  private
+    private
 
     # Automatically invoke `bundle install` and resume if
     # Bundler.settings[:auto_install] exists. This is set through config cmd
-    # `bundle config auto_install 1`.
+    # `bundle config set --global auto_install 1`.
     #
     # Note that this method `nil`s out the global Definition object, so it
     # should be called first, before you instantiate anything like an
@@ -764,7 +808,7 @@ module Bundler
       return unless SharedHelpers.md5_available?
 
       latest = Fetcher::CompactIndex.
-               new(nil, Source::Rubygems::Remote.new(URI("https://rubygems.org")), nil).
+               new(nil, Source::Rubygems::Remote.new(Bundler::URI("https://rubygems.org")), nil).
                send(:compact_index_client).
                instance_variable_get(:@cache).
                dependencies("bundler").
@@ -788,5 +832,34 @@ module Bundler
     rescue RuntimeError
       nil
     end
+
+    def remembered_negative_flag_deprecation(name)
+      positive_name = name.gsub(/\Ano-/, "")
+      option = current_command.options[positive_name]
+      flag_name = "--no-" + option.switch_name.gsub(/\A--/, "")
+
+      flag_deprecation(positive_name, flag_name, option)
+    end
+
+    def remembered_flag_deprecation(name)
+      option = current_command.options[name]
+      flag_name = option.switch_name
+
+      flag_deprecation(name, flag_name, option)
+    end
+
+    def flag_deprecation(name, flag_name, option)
+      name_index = ARGV.find {|arg| flag_name == arg.split("=")[0] }
+      return unless name_index
+
+      value = options[name]
+      value = value.join(" ").to_s if option.type == :array
+
+      Bundler::SharedHelpers.major_deprecation 2,
+        "The `#{flag_name}` flag is deprecated because it relies on being " \
+        "remembered across bundler invocations, which bundler will no longer " \
+        "do in future versions. Instead please use `bundle config set --local #{name.tr("-", "_")} " \
+        "'#{value}'`, and stop using this flag"
+    end
   end
 end
