@@ -1,4 +1,5 @@
 # frozen_string_literal: true
+
 module Bundler
   class BundlerError < StandardError
     def self.status_code(code)
@@ -54,6 +55,8 @@ module Bundler
   class PluginError < BundlerError; status_code(29); end
   class SudoNotPermittedError < BundlerError; status_code(30); end
   class ThreadCreationError < BundlerError; status_code(33); end
+  class APIResponseMismatchError < BundlerError; status_code(34); end
+  class APIResponseInvalidDependenciesError < BundlerError; status_code(35); end
   class GemfileEvalError < GemfileError; end
   class MarshalError < StandardError; end
 
@@ -137,7 +140,7 @@ module Bundler
   class NoSpaceOnDeviceError < PermissionError
     def message
       "There was an error while trying to #{action} `#{@path}`. " \
-      "There was insufficent space remaining on the device."
+      "There was insufficient space remaining on the device."
     end
 
     status_code(31)
